@@ -15,7 +15,7 @@
 
 #define PORT 8888
 
-#define REALM     "\"Maintenance\""
+#define REALM     "Maintenance"
 #define USER      "a legitimate user"
 #define PASSWORD  "and his password"
 
@@ -23,44 +23,7 @@
 #define SERVERCERTFILE "server.pem"
 
 
-static char *
-string_to_base64 (const char *message)
-{
-  const char *lookup =
-    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-  unsigned long l;
-  size_t i;
-  char *tmp;
-  size_t length = strlen (message);
-
-  tmp = malloc (length * 2 + 1);
-  if (NULL == tmp)
-    return NULL;
-  tmp[0] = 0;
-  for (i = 0; i < length; i += 3)
-  {
-    l = (((unsigned long) message[i]) << 16)
-        | (((i + 1) < length) ? (((unsigned long) message[i + 1]) << 8) : 0)
-        | (((i + 2) < length) ? ((unsigned long) message[i + 2]) : 0);
-
-
-    strncat (tmp, &lookup[(l >> 18) & 0x3F], 1);
-    strncat (tmp, &lookup[(l >> 12) & 0x3F], 1);
-
-    if (i + 1 < length)
-      strncat (tmp, &lookup[(l >> 6) & 0x3F], 1);
-    if (i + 2 < length)
-      strncat (tmp, &lookup[l & 0x3F], 1);
-  }
-
-  if (length % 3)
-    strncat (tmp, "===", 3 - length % 3);
-
-  return tmp;
-}
-
-
-static long
+static size_t
 get_file_size (const char *filename)
 {
   FILE *fp;
@@ -75,7 +38,7 @@ get_file_size (const char *filename)
 
     fclose (fp);
 
-    return size;
+    return (size_t) size;
   }
   else
     return 0;
@@ -87,7 +50,7 @@ load_file (const char *filename)
 {
   FILE *fp;
   char *buffer;
-  long size;
+  size_t size;
 
   size = get_file_size (filename);
   if (0 == size)
@@ -105,7 +68,7 @@ load_file (const char *filename)
   }
   buffer[size] = '\0';
 
-  if (size != (long) fread (buffer, 1, size, fp))
+  if (size != fread (buffer, 1, size, fp))
   {
     free (buffer);
     buffer = NULL;
@@ -121,36 +84,15 @@ ask_for_authentication (struct MHD_Connection *connection, const char *realm)
 {
   enum MHD_Result ret;
   struct MHD_Response *response;
-  char *headervalue;
-  size_t slen;
-  const char *strbase = "Basic realm=";
 
-  response = MHD_create_response_from_buffer (0, NULL,
-                                              MHD_RESPMEM_PERSISTENT);
+  response = MHD_create_response_empty (MHD_RF_NONE);
   if (! response)
     return MHD_NO;
 
-  slen = strlen (strbase) + strlen (realm) + 1;
-  if (NULL == (headervalue = malloc (slen)))
-    return MHD_NO;
-  snprintf (headervalue,
-            slen,
-            "%s%s",
-            strbase,
-            realm);
-  ret = MHD_add_response_header (response,
-                                 "WWW-Authenticate",
-                                 headervalue);
-  free (headervalue);
-  if (! ret)
-  {
-    MHD_destroy_response (response);
-    return MHD_NO;
-  }
-
-  ret = MHD_queue_response (connection,
-                            MHD_HTTP_UNAUTHORIZED,
-                            response);
+  ret = MHD_queue_basic_auth_required_response3 (connection,
+                                                 realm,
+                                                 MHD_YES,
+                                                 response);
   MHD_destroy_response (response);
   return ret;
 }
@@ -161,37 +103,23 @@ is_authenticated (struct MHD_Connection *connection,
                   const char *username,
                   const char *password)
 {
-  const char *headervalue;
-  char *expected_b64;
-  char *expected;
-  const char *strbase = "Basic ";
+  struct MHD_BasicAuthInfo *auth_info;
   int authenticated;
-  size_t slen;
 
-  headervalue =
-    MHD_lookup_connection_value (connection, MHD_HEADER_KIND,
-                                 "Authorization");
-  if (NULL == headervalue)
+  auth_info = MHD_basic_auth_get_username_password3 (connection);
+  if (NULL == auth_info)
     return 0;
-  if (0 != strncmp (headervalue, strbase, strlen (strbase)))
-    return 0;
-
-  slen = strlen (username) + 1 + strlen (password) + 1;
-  if (NULL == (expected = malloc (slen)))
-    return 0;
-  snprintf (expected,
-            slen,
-            "%s:%s",
-            username,
-            password);
-  expected_b64 = string_to_base64 (expected);
-  free (expected);
-  if (NULL == expected_b64)
-    return 0;
-
   authenticated =
-    (strcmp (headervalue + strlen (strbase), expected_b64) == 0);
-  free (expected_b64);
+    ( (strlen (username) == auth_info->username_len) &&
+      (0 == memcmp (auth_info->username, username, auth_info->username_len)) &&
+      /* The next check against NULL is optional,
+       * if 'password' is NULL then 'password_len' is always zero. */
+      (NULL != auth_info->password) &&
+      (strlen (password) == auth_info->password_len) &&
+      (0 == memcmp (auth_info->password, password, auth_info->password_len)) );
+
+  MHD_free (auth_info);
+
   return authenticated;
 }
 
@@ -203,9 +131,7 @@ secret_page (struct MHD_Connection *connection)
   struct MHD_Response *response;
   const char *page = "<html><body>A secret.</body></html>";
 
-  response =
-    MHD_create_response_from_buffer (strlen (page), (void *) page,
-                                     MHD_RESPMEM_PERSISTENT);
+  response = MHD_create_response_from_buffer_static (strlen (page), page);
   if (! response)
     return MHD_NO;
 
@@ -220,7 +146,7 @@ static enum MHD_Result
 answer_to_connection (void *cls, struct MHD_Connection *connection,
                       const char *url, const char *method,
                       const char *version, const char *upload_data,
-                      size_t *upload_data_size, void **con_cls)
+                      size_t *upload_data_size, void **req_cls)
 {
   (void) cls;               /* Unused. Silent compiler warning. */
   (void) url;               /* Unused. Silent compiler warning. */
@@ -230,9 +156,9 @@ answer_to_connection (void *cls, struct MHD_Connection *connection,
 
   if (0 != strcmp (method, "GET"))
     return MHD_NO;
-  if (NULL == *con_cls)
+  if (NULL == *req_cls)
   {
-    *con_cls = connection;
+    *req_cls = connection;
     return MHD_YES;
   }
 
@@ -244,7 +170,7 @@ answer_to_connection (void *cls, struct MHD_Connection *connection,
 
 
 int
-main ()
+main (void)
 {
   struct MHD_Daemon *daemon;
   char *key_pem;
