@@ -1,6 +1,7 @@
 /*
      This file is part of libmicrohttpd
      Copyright (C) 2007, 2008 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2016-2022 Evgeny Grin (Karlson2k)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -21,6 +22,7 @@
  * @brief example for how to get the query string from libmicrohttpd
  *        Call with an URI ending with something like "?q=QUERY"
  * @author Christian Grothoff
+ * @author Karlson2k (Evgeny Grin)
  */
 
 #include "platform.h"
@@ -35,14 +37,16 @@ ahc_echo (void *cls,
           const char *url,
           const char *method,
           const char *version,
-          const char *upload_data, size_t *upload_data_size, void **ptr)
+          const char *upload_data, size_t *upload_data_size, void **req_cls)
 {
   static int aptr;
-  const char *fmt = cls;
   const char *val;
   char *me;
   struct MHD_Response *response;
   enum MHD_Result ret;
+  int resp_len;
+  size_t buf_size;
+  (void) cls;               /* Unused. Silent compiler warning. */
   (void) url;               /* Unused. Silent compiler warning. */
   (void) version;           /* Unused. Silent compiler warning. */
   (void) upload_data;       /* Unused. Silent compiler warning. */
@@ -50,20 +54,32 @@ ahc_echo (void *cls,
 
   if (0 != strcmp (method, "GET"))
     return MHD_NO;              /* unexpected method */
-  if (&aptr != *ptr)
+  if (&aptr != *req_cls)
   {
     /* do never respond on first call */
-    *ptr = &aptr;
+    *req_cls = &aptr;
     return MHD_YES;
   }
-  *ptr = NULL;                  /* reset when done */
+  *req_cls = NULL;  /* reset when done */
   val = MHD_lookup_connection_value (connection, MHD_GET_ARGUMENT_KIND, "q");
-  me = malloc (snprintf (NULL, 0, fmt, "q", val) + 1);
+  if (NULL == val)
+    return MHD_NO;  /* No "q" argument was found */
+  resp_len = snprintf (NULL, 0, PAGE, "q", val);
+  if (0 >= resp_len)
+    return MHD_NO;  /* Error calculating response size */
+  buf_size = (size_t) resp_len + 1; /* Add one byte for zero-termination */
+  me = malloc (buf_size);
   if (me == NULL)
-    return MHD_NO;
-  sprintf (me, fmt, "q", val);
-  response = MHD_create_response_from_buffer (strlen (me), me,
-                                              MHD_RESPMEM_MUST_FREE);
+    return MHD_NO;  /* Error allocating memory */
+  if (resp_len != snprintf (me, buf_size, PAGE, "q", val))
+  {
+    free (me);
+    return MHD_NO;  /* Error forming the response body */
+  }
+  response =
+    MHD_create_response_from_buffer_with_free_callback (buf_size - 1,
+                                                        (void *) me,
+                                                        &free);
   if (response == NULL)
   {
     free (me);
@@ -96,7 +112,7 @@ main (int argc, char *const *argv)
   d = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION
                         | MHD_USE_INTERNAL_POLLING_THREAD | MHD_USE_ERROR_LOG,
                         (uint16_t) port,
-                        NULL, NULL, &ahc_echo, PAGE, MHD_OPTION_END);
+                        NULL, NULL, &ahc_echo, NULL, MHD_OPTION_END);
   if (NULL == d)
     return 1;
   (void) getc (stdin);
