@@ -1,7 +1,8 @@
 /*
      This file is part of libmicrohttpd
-     Copyright (C) 2016, 2017 Christian Grothoff,
-     Silvio Clecio (silvioprog), Karlson2k (Evgeny Grin)
+     Copyright (C) 2016-2017 Christian Grothoff,
+     Silvio Clecio (silvioprog), Evgeny Grin (Karlson2k)
+     Copyright (C) 2022 Evgeny Grin (Karlson2k)
 
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Lesser General Public
@@ -37,7 +38,7 @@ answer_to_connection (void *cls,
                       const char *version,
                       const char *upload_data,
                       size_t *upload_data_size,
-                      void **con_cls)
+                      void **req_cls)
 {
   const char *page = "<html><body>Hello timeout!</body></html>";
   struct MHD_Response *response;
@@ -48,14 +49,18 @@ answer_to_connection (void *cls,
   (void) method;            /* Unused. Silent compiler warning. */
   (void) upload_data;       /* Unused. Silent compiler warning. */
   (void) upload_data_size;  /* Unused. Silent compiler warning. */
-  (void) con_cls;           /* Unused. Silent compiler warning. */
+  (void) req_cls;           /* Unused. Silent compiler warning. */
 
-  response = MHD_create_response_from_buffer (strlen (page),
-                                              (void *) page,
-                                              MHD_RESPMEM_PERSISTENT);
-  MHD_add_response_header (response,
-                           MHD_HTTP_HEADER_CONTENT_TYPE,
-                           "text/html");
+  response = MHD_create_response_from_buffer_static (strlen (page),
+                                                     (const void *) page);
+  if (MHD_YES !=
+      MHD_add_response_header (response,
+                               MHD_HTTP_HEADER_CONTENT_TYPE,
+                               "text/html"))
+  {
+    fprintf (stderr,
+             "Failed to set content type header!\n");
+  }
   ret = MHD_queue_response (connection,
                             MHD_HTTP_OK,
                             response);
