@@ -1,6 +1,7 @@
 /*
      This file is part of libmicrohttpd
      Copyright (C) 2007, 2009, 2011 Christian Grothoff
+     Copyright (C) 2014-2022 Evgeny Grin (Karlson2k)
 
      libmicrohttpd is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -22,11 +23,13 @@
  * @brief Testcase for MHD not calling the callback too often
  * @author Jan Seeger
  * @author Christian Grothoff
+ * @author Karlson2k (Evgeny Grin)
  */
 #include "MHD_config.h"
 #include "platform.h"
 #include <curl/curl.h>
 #include <microhttpd.h>
+#include <errno.h>
 
 struct callback_closure
 {
@@ -45,7 +48,7 @@ called_twice (void *cls, uint64_t pos, char *buf, size_t max)
   {
     memcpy (buf, "test", 5);
     cls2->called = 1;
-    return strlen (buf);
+    return (ssize_t) strlen (buf);
   }
   if (cls2->called == 1)
   {
@@ -54,6 +57,7 @@ called_twice (void *cls, uint64_t pos, char *buf, size_t max)
   }
   fprintf (stderr,
            "Handler called after returning END_OF_STREAM!\n");
+  abort ();
   return MHD_CONTENT_READER_END_WITH_ERROR;
 }
 
@@ -66,7 +70,7 @@ callback (void *cls,
           const char *version,
           const char *upload_data,
           size_t *upload_data_size,
-          void **con_cls)
+          void **req_cls)
 {
   struct callback_closure *cbc = calloc (1, sizeof(struct callback_closure));
   struct MHD_Response *r;
@@ -78,7 +82,7 @@ callback (void *cls,
   (void) version;
   (void) upload_data; /* Unused. Silent compiler warning. */
   (void) upload_data_size;
-  (void) con_cls;         /* Unused. Silent compiler warning. */
+  (void) req_cls;         /* Unused. Silent compiler warning. */
 
   if (NULL == cbc)
     return MHD_NO;
@@ -129,7 +133,7 @@ main (int argc, char **argv)
   int running;
   struct timeval tv;
   int extra;
-  int port;
+  uint16_t port;
   (void) argc; (void) argv; /* Unused. Silent compiler warning. */
 
   if (MHD_NO != MHD_is_feature_supported (MHD_FEATURE_AUTODETECT_BIND_PORT))
@@ -137,12 +141,13 @@ main (int argc, char **argv)
   else
     port = 1140;
 
-  d = MHD_start_daemon (0,
+  d = MHD_start_daemon (MHD_USE_NO_THREAD_SAFETY,
                         port,
                         NULL,
                         NULL,
                         &callback,
                         NULL,
+                        MHD_OPTION_APP_FD_SETSIZE, (int) FD_SETSIZE,
                         MHD_OPTION_END);
   if (d == NULL)
     return 32;
@@ -154,7 +159,7 @@ main (int argc, char **argv)
     {
       MHD_stop_daemon (d); return 48;
     }
-    port = (int) dinfo->port;
+    port = dinfo->port;
   }
   c = curl_easy_init ();
   curl_easy_setopt (c, CURLOPT_URL, "http://127.0.0.1/");
@@ -170,7 +175,7 @@ main (int argc, char **argv)
   {
     curl_easy_cleanup (c);
     MHD_stop_daemon (d);
-    return 1;
+    return 99;
   }
   mret = curl_multi_add_handle (multi, c);
   if (mret != CURLM_OK)
@@ -178,7 +183,7 @@ main (int argc, char **argv)
     curl_multi_cleanup (multi);
     curl_easy_cleanup (c);
     MHD_stop_daemon (d);
-    return 2;
+    return 99;
   }
   extra = 10;
   while ( (c != NULL) || (--extra > 0) )
@@ -198,7 +203,7 @@ main (int argc, char **argv)
         curl_multi_cleanup (multi);
         curl_easy_cleanup (c);
         MHD_stop_daemon (d);
-        return 3;
+        return 99;
       }
     }
     if (MHD_YES !=
@@ -216,37 +221,58 @@ main (int argc, char **argv)
     {
 #ifdef MHD_POSIX_SOCKETS
       if (EINTR != errno)
-        abort ();
+      {
+        fprintf (stderr, "Unexpected select() error: %d. Line: %d\n",
+                 (int) errno, __LINE__);
+        fflush (stderr);
+        exit (99);
+      }
 #else
-      if ((WSAEINVAL != WSAGetLastError ()) || (0 != rs.fd_count) || (0 !=
-                                                                      ws.
-                                                                      fd_count)
-          || (0 != es.fd_count) )
-        abort ();
-      Sleep (1000);
+      if ((WSAEINVAL != WSAGetLastError ()) ||
+          (0 != rs.fd_count) || (0 != ws.fd_count) || (0 != es.fd_count) )
+      {
+        fprintf (stderr, "Unexpected select() error: %d. Line: %d\n",
+                 (int) WSAGetLastError (), __LINE__);
+        fflush (stderr);
+        exit (99);
+      }
+      Sleep (1);
 #endif
     }
     if (NULL != multi)
     {
       curl_multi_perform (multi, &running);
-      if (running == 0)
+      if (0 == running)
       {
-        msg = curl_multi_info_read (multi, &running);
-        if (msg == NULL)
-          break;
-        if (msg->msg == CURLMSG_DONE)
+        int pending;
+        int curl_fine = 0;
+        while (NULL != (msg = curl_multi_info_read (multi, &pending)))
         {
-          if (msg->data.result != CURLE_OK)
-            printf ("%s failed at %s:%d: `%s'\n",
-                    "curl_multi_perform",
-                    __FILE__,
-                    __LINE__, curl_easy_strerror (msg->data.result));
-          curl_multi_remove_handle (multi, c);
-          curl_multi_cleanup (multi);
-          curl_easy_cleanup (c);
-          c = NULL;
-          multi = NULL;
+          if (msg->msg == CURLMSG_DONE)
+          {
+            if (msg->data.result == CURLE_OK)
+              curl_fine = 1;
+            else
+            {
+              fprintf (stderr,
+                       "%s failed at %s:%d: `%s'\n",
+                       "curl_multi_perform",
+                       __FILE__,
+                       __LINE__, curl_easy_strerror (msg->data.result));
+              abort ();
+            }
+          }
         }
+        if (! curl_fine)
+        {
+          fprintf (stderr, "libcurl haven't returned OK code\n");
+          abort ();
+        }
+        curl_multi_remove_handle (multi, c);
+        curl_multi_cleanup (multi);
+        curl_easy_cleanup (c);
+        c = NULL;
+        multi = NULL;
       }
     }
     MHD_run (d);
