@@ -38,6 +38,7 @@
 #include <gcrypt.h>
 #endif
 #endif /* MHD_HTTPS_REQUIRE_GCRYPT */
+#include <errno.h>
 
 #ifndef WINDOWS
 #include <unistd.h>
@@ -77,15 +78,15 @@ struct CBC
 static void
 completed_cb (void *cls,
               struct MHD_Connection *connection,
-              void **con_cls,
+              void **req_cls,
               enum MHD_RequestTerminationCode toe)
 {
-  struct MHD_PostProcessor *pp = *con_cls;
+  struct MHD_PostProcessor *pp = *req_cls;
   (void) cls; (void) connection; (void) toe;            /* Unused. Silent compiler warning. */
 
   if (NULL != pp)
     MHD_destroy_post_processor (pp);
-  *con_cls = NULL;
+  *req_cls = NULL;
 }
 
 
@@ -140,7 +141,7 @@ ahc_echo (void *cls,
           const char *method,
           const char *version,
           const char *upload_data, size_t *upload_data_size,
-          void **unused)
+          void **req_cls)
 {
   static int eok;
   struct MHD_Response *response;
@@ -153,25 +154,31 @@ ahc_echo (void *cls,
     printf ("METHOD: %s\n", method);
     return MHD_NO;              /* unexpected method */
   }
-  pp = *unused;
+  pp = *req_cls;
   if (pp == NULL)
   {
     eok = 0;
-    pp = MHD_create_post_processor (connection, 1024, &post_iterator, &eok);
-    if (pp == NULL)
+    pp = MHD_create_post_processor (connection,
+                                    1024,
+                                    &post_iterator,
+                                    &eok);
+    if (NULL == pp)
       abort ();
-    *unused = pp;
+    *req_cls = pp;
   }
-  MHD_post_process (pp, upload_data, *upload_data_size);
+  if (MHD_YES !=
+      MHD_post_process (pp,
+                        upload_data,
+                        *upload_data_size))
+    abort ();
   if ((eok == 3) && (0 == *upload_data_size))
   {
-    response = MHD_create_response_from_buffer (strlen (url),
-                                                (void *) url,
-                                                MHD_RESPMEM_MUST_COPY);
+    response = MHD_create_response_from_buffer_copy (strlen (url),
+                                                     (const void *) url);
     ret = MHD_queue_response (connection, MHD_HTTP_OK, response);
     MHD_destroy_response (response);
     MHD_destroy_post_processor (pp);
-    *unused = NULL;
+    *req_cls = NULL;
     return ret;
   }
   *upload_data_size = 0;
@@ -264,8 +271,8 @@ free_test_form (struct mhd_test_postdata *postdata)
 }
 
 
-static int
-testInternalPost ()
+static unsigned int
+testInternalPost (void)
 {
   struct MHD_Daemon *d;
   CURL *c;
@@ -346,8 +353,8 @@ testInternalPost ()
 }
 
 
-static int
-testMultithreadedPost ()
+static unsigned int
+testMultithreadedPost (void)
 {
   struct MHD_Daemon *d;
   CURL *c;
@@ -429,8 +436,8 @@ testMultithreadedPost ()
 }
 
 
-static int
-testMultithreadedPoolPost ()
+static unsigned int
+testMultithreadedPoolPost (void)
 {
   struct MHD_Daemon *d;
   CURL *c;
@@ -512,8 +519,8 @@ testMultithreadedPoolPost ()
 }
 
 
-static int
-testExternalPost ()
+static unsigned int
+testExternalPost (void)
 {
   struct MHD_Daemon *d;
   CURL *c;
@@ -550,9 +557,10 @@ testExternalPost ()
   cbc.buf = buf;
   cbc.size = 2048;
   cbc.pos = 0;
-  d = MHD_start_daemon (MHD_USE_ERROR_LOG,
+  d = MHD_start_daemon (MHD_USE_ERROR_LOG | MHD_USE_NO_THREAD_SAFETY,
                         port, NULL, NULL, &ahc_echo, NULL,
                         MHD_OPTION_NOTIFY_COMPLETED, &completed_cb, NULL,
+                        MHD_OPTION_APP_FD_SETSIZE, (int) FD_SETSIZE,
                         MHD_OPTION_END);
   if (d == NULL)
     return 256;
